/*

=head1 NAME

Phasegraph - Calculate operator phase difference

=head1 SYNOPSIS

Phasegraph [pars]


=head1 Input parameters

=over 4


=item  d1 - float  

       [0.002] time step to analyze

=item   d2 - float 

        [10.0] spatial grid spacing to analyze

=item   vmin - float 

        [1500.0]  minimum velocity

=item   vmax - float

        [3000.0]  maximum velocity

=item   vinc - float 

        [100.0] Velocity increment

=back

=head1 DESCRIPTION 

Calculate phase error for one time step in sep150/huyle

=head1 CATEGORY

B<util/info>

=cut
>*/
/*
Author: Stewart A. Levin

8/26/13  Initial version
*/
#define         HEAD    "/dev/null"
#define         OUT    "/dev/null"
/*#include <sep.main>*/
#include<unistd.h>
/*#include <seplib.h>*/
#include <sitedef.h>
#include <sys/types.h>
#include <string.h>
#include <seplib.h>
#include<sep3dc.h>
#include <sys/file.h>
#include <sys/stat.h>
#include<sepmath.h>
#include<strings.h>
#include <math.h>


#if defined (HAVE_STDLIB_H)
#include<stdlib.h>
#else
extern int atoi();
#endif /* HAVE_STDLIB  */
int main (int argc, char **argv)
{
int i,n;
float d1, d2, vmin, vmax, vinc;
float outval, ftemp;
double knyq, delk, k;
double v;
double khat;
double cosk, sink, coskhat, sinkhat;
double dtemp, etemp, delphi;
double bcoefs[9] = {
   -3.18543410, 1.89789462, -0.40456799, 0.13676734, -0.05150324,
   0.01893502, -0.00619345, 0.00159455, -0.00020980 };

  initpar(argc,argv);
  getch_add_string("noheader=y");

d1=0.004;
getch("d1","f",&d1);
d2=10.0;
getch("d2","f",&d2);
vmin=1500.0;
getch("vmin","f",&vmin);
vmax=3000.0;
getch("vmax","f",&vmax);
vinc=100.0;
getch("vinc","f",&vinc);

knyq = 3.145926535/d2;
delk = knyq/101.0;
n = 102;

ftemp = (float) delk;
putch("o1","f",&ftemp);
putch("d1","f",&ftemp);
putch("n1","i", &n);
putch("label1","s","wavenumber");
putch("o2","f",&vmin);
putch("d2","f",&vinc);
n = 1+(int) ((vmax+1.0e-16-vmin)/vinc);
putch("n2","d",&n );
putch("label2","s","velocity");
ftemp = 0.0f;
putch("o3","f",&ftemp);
ftemp = 1.0f;
putch("d3","f",&ftemp);
n = 1;
putch("n3","i",&n);

for(v=vmin; v <= vmax; v += vinc) {
    for(k=0.0; k<(knyq-1.0e-16); k += delk) {
       dtemp = bcoefs[0];    
       for(i=1; i< sizeof(bcoefs)/sizeof(bcoefs[0]); ++i) {
          dtemp += 2.0*bcoefs[i]*cos(i*k*d2);
       }
       khat = sqrt(-dtemp)/d2;
       cosk = 0.5*(2.0-v*v*d1*d1*k*k);         
       sink = sqrt(1.0-cosk*cosk);
       coskhat = 0.5*(2.0-v*v*d1*d1*khat*khat);         
       if(-1.0 <= coskhat && coskhat <= 1.0) {
       sinkhat = sqrt(1.0-coskhat*coskhat);
       dtemp = coskhat*cosk+sinkhat*sink;
       etemp = coskhat*sink-sinkhat*cosk;
       delphi = atan2(etemp,dtemp);
       /*delphi = log(10000.0*delphi+1.0);*/
       outval = (float) delphi;
       } else { outval = 0.0f; }
       srite("out",&outval,sizeof(outval));
    }
}

return(0);
}
