module modeling_mod_old

  implicit none
  
  real, private :: ds, dt, v, a, f, dk
  real, parameter :: pi = 3.14159265359
  integer, private :: nx, nz, nt, m, nv
  real :: c2(0:1), c4(0:2), c6(0:3), c8(0:4), c10(0:5), c12(0:6), c14(0:7), c16(0:8)
  real, allocatable, private ::vv(:), pv(:)


  contains 

  subroutine init_c16()
    ! Calculate coefficients of 16th order finite difference
    	  
    c16(1) = 1.
    c16(2) = -7./40.
    c16(3) = 7./165.
    c16(4) = -7./704.
    c16(5) = 7./3575.
    c16(6) = -1./3432.
    c16(7) = 1./35035.
    c16(8) = -1./732160.
    c16(0) = -2*sum(c16(1:8))
    c16 = 16./9.*c16
	  
  end subroutine

  subroutine init_c14()
    ! Calculate coefficients of 14th order finite difference

    c14(1) = 1.
    c14(2) = -1./6.
    c14(3) = 1./27.
    c14(4) = -1./132.
    c14(5) = 1./825.
    c14(6) = -1./7722.
    c14(7) = -1./147147.
    c14(0) = -2*sum(c14(1:7))
    c14 = 7./4.*c14
	  
  end subroutine

  subroutine init_c12()
    ! Calculate coefficients of 12th order finite difference

    c12(1) = 1.
    c12(2) = -5./32.
    c12(3) = 5./162.
    c12(4) = -1./192.
    c12(5) = 1./1650.
    c12(6) = -1./28512.
    c12(0) = -2*sum(c12(1:6))
    c12 = 12./7.*c12
	  
  end subroutine

  subroutine init_c10()
    ! Calculate coefficients of 10th order finite difference

    c10(1) = 1.
    c10(2) = -1./7.
    c10(3) = 1./42.
    c10(4) = -1./336.
    c10(5) = 1./5250.
    c10(0) = -2*sum(c10(1:5))
    c10 = 5./3.*c10
	  
  end subroutine

  subroutine init_c8()
    ! Calculate coefficients of 8th order finite difference

    c8(1) = 1.
    c8(2) = -1./8.
    c8(3) = 1./63.
    c8(4) = -1./896.
    c8(0) = -2*sum(c8(1:4))
    c8 = 8./5.*c8
	  
  end subroutine

  subroutine init_myc8()
    ! Calculate coefficients of 8th order finite difference

    c8(1) = 1.755863
    c8(2) = -0.290179
    c8(3) = 0.056295
    c8(4) = -0.0063626133
    c8(0) = -2*sum(c8(1:4))
	  
  end subroutine

  subroutine init_paperc8()
    ! Calculate coefficients of 8th order finite difference

    c8(1) = 1.70507669
    c8(2) = -0.25861812
    c8(3) = 0.04577745
    c8(4) = -0.0052363
    c8(0) = -2*sum(c8(1:4))
	  
  end subroutine

  subroutine init_c6()
    ! Calculate coefficients of 6th order finite difference

    c6(1) = 1.
    c6(2) = -1./10.
    c6(3) = 1./135.
    c6(0) = -2*sum(c6(1:3))
    c6 = 3./2.*c6
	  
  end subroutine

  subroutine init_c4()
    ! Calculate coefficients of 4th order finite difference

    c4(1) = 1.
    c4(2) = -1./16.
    c4(0) = -2*sum(c4(1:2))
    c4 = 4./3.*c4
	  
  end subroutine

  subroutine init_myc4()
    ! Calculate coefficients of 4th order finite difference

    c4(1) = 1.3662928
    c4(2) = -0.0915732
    c4(0) = -2*sum(c4(1:2))
	  
  end subroutine

  subroutine init_paperc4()
    ! Calculate coefficients of 4th order finite difference

    c4(1) = 1.37106192
    c4(2) = -0.09322459
    c4(0) = -2*sum(c4(1:2))
	  
  end subroutine

  subroutine init(ds_in, dt_in, nx_in, nz_in, v_in)
  ! Initialize grid dimensions, time and spatial steps for the module

    real :: ds_in, dt_in, v_in
    integer :: nx_in, nz_in
	
      ds = ds_in
      dt = dt_in
      nx = nx_in
      nz = nz_in
      v = v_in
      a = dt*dt*v*v/ds/ds

  end subroutine  

  subroutine one_step4(prev, cur, next, c)
    real :: prev(nx,nz), cur(nx,nz), next(nx,nz), c(0:2)
    integer :: ix, iz

    do iz=3,nz-2
      do ix=3,nx-2
        next(ix,iz) = a*(c(0)*2*cur(ix,iz) &
                        +c(1)*(cur(ix-1,iz)+cur(ix+1,iz)+cur(ix,iz-1)+cur(ix,iz+1)) &
                        +c(2)*(cur(ix-2,iz)+cur(ix+2,iz)+cur(ix,iz-2)+cur(ix,iz+2))) &
                      - prev(ix,iz) + 2*cur(ix,iz)
      enddo
    enddo

  end subroutine

  subroutine one_step8(prev, cur, next, c)
    real :: prev(nx,nz), cur(nx,nz), next(nx,nz), c(0:4)
    integer :: ix, iz
    write(0,*) c
    do iz=3,nz-2
      do ix=3,nx-2
        next(ix,iz) = a*(c(0)*2*cur(ix,iz) &
                        +c(1)*(cur(ix-1,iz)+cur(ix+1,iz)+cur(ix,iz-1)+cur(ix,iz+1)) &
                        +c(2)*(cur(ix-2,iz)+cur(ix+2,iz)+cur(ix,iz-2)+cur(ix,iz+2)) &
                        +c(3)*(cur(ix-3,iz)+cur(ix+3,iz)+cur(ix,iz-3)+cur(ix,iz+3)) &
                        +c(4)*(cur(ix-4,iz)+cur(ix+4,iz)+cur(ix,iz-4)+cur(ix,iz+4))) &
                      - prev(ix,iz) + 2*cur(ix,iz)
      enddo
    enddo

  end subroutine

  double precision function p(ei,ee,t)
 
    double precision, intent(in) :: ei,ee, t
  
      if (ee < ei) then
        p = 1.
      else
        p = exp((ei-ee)/t)
      endif

  end function

  subroutine oper_init(m_in, ds_in, v_in, f_in,pfit_in)

    real, intent(in) :: ds_in, v_in, f_in, pfit_in
    integer, intent(in) :: m_in

    m = m_in
    ds = ds_in
    dk = min(pfit_in*pi/ds/100.,pfit_in*2*pi*f_in/v_in/100.)

  end subroutine

  double precision function maxe(a)

    real, intent(in) :: a(m)
    double precision :: temp, k(100)
    integer :: i, j

    k = (/ (i*dk, i=1,100) /)

    maxe = 0.

    do i=1,100
      temp = ds*ds*k(i)*k(i)
      do j=1,m
        temp = temp + 2*a(j)*(cos(j*k(i)*ds)-1)
      enddo
      if (abs(temp) > maxe) then
        maxe = abs(temp)
      endif
    enddo

  end function
 
  double precision function maxev(a,v1)

    real, intent(in) :: a(m), v1
    double precision :: temp, k(100)
    integer :: i, j
    real :: dk1

    dk1 = 2*pi*f/v1/100.
    k = (/ (i*dk1, i=1,100) /)

    maxev = 0.

    do i=1,100
      temp = ds*ds*k(i)*k(i)
      do j=1,m
        temp = temp + 2*a(j)*(cos(j*k(i)*ds)-1)
      enddo
      if (abs(temp) > maxev) then
        maxev = abs(temp)
      endif
    enddo

  end function

  subroutine oper_initp(m_in, ds_in, v_in, pv_in, f_in,pfit_in, nv_in)

    real, intent(in) :: ds_in, f_in, pfit_in, v_in(:), pv_in(:)
    integer, intent(in) :: m_in, nv_in

    nv = nv_in
    allocate(vv(nv), pv(nv))
    vv = v_in
    pv = pv_in
    m = m_in
    ds = ds_in
    f = f_in
    dk = min(pfit_in*pi/ds/100.,pfit_in*2*pi*f_in/minval(v_in)/100.)

  end subroutine
 
  double precision function errp(a)

    real, intent(in) :: a(m)
    integer :: i

    errp = 0.

    do i=1,nv
      errp = errp + maxev(a,vv(i))*pv(i)
    enddo

    errp = errp/sum(pv)

  end function

  subroutine neighbor(a,an)
  
    real :: a(m), an(m), r
    integer :: i

    do while (.true.)
      call random_number(r)
      an(2) = a(2) + 0.01*c8(2)*(r-0.5)
      if (an(2) < 0.) then
        exit
      endif
    enddo

    do while (.true.)
      call random_number(r)
      an(3) = a(3) + 0.01*c8(3)*(r-0.5)
      if (an(3) > 0. .and. abs(an(3)) <= abs(an(2))) then
        exit
      endif
    enddo

    do while (.true.)
      call random_number(r)
      an(4) = a(4) + 0.01*c8(4)*(r-0.5)
      if (an(4) < 0. .and. abs(an(4)) <= abs(an(3))) then
        exit
      endif
    enddo

    an(1) = 1.
    do i=2,m
      an(1) = an(1) - i*i*an(i)
    enddo

  end subroutine 

  subroutine neighborp(a,an)
  
    real :: a(m), an(m), r

    do while (.true.)
      call random_number(r)
      an(1) = a(1) + 0.01*c8(1)*(r-0.5)
      if (an(1) > 0.) then
        exit
      endif
    enddo

    do while (.true.)
      call random_number(r)
      an(2) = a(2) + 0.01*c8(2)*(r-0.5)
      if (an(2) < 0. .and. abs(an(2)) <= abs(an(1))) then
        exit
      endif
    enddo

    do while (.true.)
      call random_number(r)
      an(3) = a(3) + 0.01*c8(3)*(r-0.5)
      if (an(3) > 0. .and. abs(an(3)) <= abs(an(2))) then
        exit
      endif
    enddo

    do while (.true.)
      call random_number(r)
      an(4) = a(4) + 0.01*c8(4)*(r-0.5)
      if (an(4) < 0. .and. abs(an(4)) <= abs(an(3))) then
        exit
      endif
    enddo

  end subroutine 

end module



