 program modeling_main

  use modeling_mod
  use sep
  use ieee_arithmetic
  use ieee_exceptions

  implicit none
  
  real :: ds, dt, c(0:4)
  real, pointer :: cur(:,:), prev(:,:), next(:,:), sou(:), field(:,:,:), vel(:,:)
  integer :: nx, nz, nt, ns, it, order

  call sep_init() 

  call from_aux("vel", "n1", nz)
  call from_aux("vel", "n2", nx)
  call from_aux("vel", "d1", ds)
  call from_param("a",c(1:4))
  c(0) = -2*sum(c(1:4))
  call from_param("order",order,2)

  ! Write out header of output wave field
  call to_history("n1",nz)
  call to_history("n2",nx)
  call to_history("d1",ds)
  call to_history("d2",ds)
  call to_history("o1",0)
  call to_history("o2",0)

  call to_history("n1",nz,"snap")
  call to_history("n2",nx,"snap")
  call to_history("d1",ds,"snap")
  call to_history("d2",ds,"snap")
  call to_history("o1",0,"snap")
  call to_history("o2",0,"snap")

  ! Allocate and initialize previous, current, and next wavefield
  allocate(vel(nz,nx), cur(nz,nx), prev(nz,nx), next(nz,nx))

  call sep_read(vel,"vel")

  dt = 0.001 
  nt = floor(3./4.*max(nx,nz)*ds/2/((maxval(vel)+minval(vel))/2.)/dt)

  allocate(field(nz,nx,nt))

  call to_history("n3",nt)
  call to_history("d3",dt)
  call to_history("o3",0)

  cur = 0.
  prev = 0.
  next = 0.

  call init(ds, dt, nx, nz)
  vel2 = vel*vel
  call init_c(c)

  call from_aux("source", "n1", ns)
  allocate(sou(ns))
  call sep_read(sou, "source")

 if (order == 2) then
  do it=1,nt
    write(0,*) it

    if (it <= ns) then
      cur(nz/2,nx/2) = cur(nz/2,nx/2) + sou(it) ! Source injection
    endif

    call one_step8(prev, cur, next)

    if (maxval(abs(next)) > 1.e17) then
      write(0,*) "blow up"
      write(0,*) maxval(abs(next))
      exit  
    endif

    if (.not. ieee_is_finite(sum(next))) then
      write(0,*) "nan"
      exit 
    endif

    prev = cur 
    cur = next
    field(:,:,it) = next

  enddo

 else

  do it=1,nt
    write(0,*) it

    if (it <= ns) then
      cur(nz/2,nx/2) = cur(nz/2,nx/2) + sou(it) ! Source injection
    endif

    call one_step84(prev, cur, next)

    if (maxval(abs(next)) > 1.e17) then
      write(0,*) "blow up"
      write(0,*) maxval(abs(next))
      exit  
    endif

    if (.not. ieee_is_finite(sum(next))) then
      write(0,*) "nan"
      exit 
    endif

    prev = cur 
    cur = next
    field(:,:,it) = next

  enddo

 endif

  call sep_write(field)
  call sep_write(field(:,:,nt),"snap")
 
  deallocate(cur, prev, next, field, sou)
  call clean_up()
  call sep_close() 

end program
