module sa_mod

  implicit none
  
  real, private :: ds, f_fun, v_mean, v_stdd, dk
  integer, private :: b, e
  real, parameter :: pi = 3.14159265359
  double precision, private :: pe(101,101), k(101,101)
  real :: c8(0:4)

  contains 

  subroutine init_paperc8()

    c8(1) = 1.70507669
    c8(2) = -0.25861812
    c8(3) = 0.04577745
    c8(4) = -0.0052363
    c8(0) = -2*sum(c8(1:4))
	  
  end subroutine

  subroutine init_c8()

    c8(1) = 1.
    c8(2) = -1./8.
    c8(3) = 1./63.
    c8(4) = -1./896.
    c8(0) = -2*sum(c8(1:4))
    c8 = 8./5.*c8
	  
  end subroutine

  double precision function p(ei,ee,t)
 
    double precision, intent(in) :: ei,ee, t

      if (ee < ei) then
        p = 1.
      else
        p = exp((ei-ee)/t)
      endif

  end function

  double precision function L2_err(a)

    real, intent(in) :: a(4)
    
    L2_err = sum(pe(1:e,b:101)*(2*(a(1)*(cos(1.*k(1:e,b:101)*ds)-1.) + a(2)*(cos(2.*k(1:e,b:101)*ds)-1.) &
                                 + a(3)*(cos(3.*k(1:e,b:101)*ds)-1.) + a(4)*(cos(4.*k(1:e,b:101)*ds)-1.)) &
                                 + ds*ds*k(1:e,b:101)**2)**2)

  end function

  double precision function max_err(a)

    real, intent(in) :: a(4)

    max_err = maxval(abs(pe(1:e,b:101) &
              *(2*(a(1)*(cos(1.*k(1:e,b:101)*ds)-1.) + a(2)*(cos(2.*k(1:e,b:101)*ds)-1.)   &                
                 + a(3)*(cos(3.*k(1:e,b:101)*ds)-1.) + a(4)*(cos(4.*k(1:e,b:101)*ds)-1.)) &
              + ds*ds*k(1:e,b:101)**2)))

  end function

  subroutine init_sa_fv(p_fit_in,ds_in,f_fun_in,v_mean_in,v_stdd_in,f_weight_in,v_weight_in)

    real, intent(in) :: p_fit_in, ds_in, f_fun_in, v_mean_in, v_stdd_in
    integer, intent(in) :: f_weight_in, v_weight_in
    integer :: i, j
    real :: df, dv, v(101), f(101), pv(101), pf(101)

    v_mean = v_mean_in
    v_stdd = v_stdd_in
    f_fun = f_fun_in
    b = 101 - nint(p_fit_in)
    e = int(p_fit_in)
    ds = ds_in

    df = 2.5*f_fun/100.
    f = (/ ((i-1)*df, i=1,101) /)
    if (f_weight_in == 0) then
      pf = 1.
    elseif (f_weight_in == 1) then
      pf = (/ (2/sqrt(pi)*f(i)**2/f_fun**3*exp(-f(i)**2/f_fun**2), i=1,101) /)
    elseif (f_weight_in == 2) then
      pf = (/ (2/sqrt(pi)*f(i)**2/f_fun**3*exp(-f(i)**2/f_fun**2), i=1,101) /)
      pf = 1. + maxval(pf) - pf
    else 
      pf = f
    endif
    
    dv = 6*v_stdd/100.
    v = (/ (v_mean-3*v_stdd+(j-1)*dv, j=1,101) /)
    if (v_weight_in == 0) then
      pv = 1.
    elseif (v_weight_in == 1) then
      pv = (/ (exp(-(v(j)-v_mean)**2/2/v_stdd**2)/sqrt(2*pi*v_stdd**2), j=1,101) /)
    elseif (v_weight_in == 2) then
      pv = (/ (exp(-(v(j)-v_mean)**2/2/v_stdd**2)/sqrt(2*pi*v_stdd**2), j=1,101) /)
      pv = 1. + maxval(pv) - pv
    else
      pv = 1./v/v
    endif

    do j=1,101
      do i=1,101
        k(i,j) = 2*pi*f(i)/v(j)
        pe(i,j) = pf(i)*pv(j)
      enddo
    enddo
 
!    pe = pe/sum(pe)

  end subroutine

  subroutine init_sa_k(p_fit_in,ds_in,v_in,f_max_in)

    real, intent(in) :: p_fit_in, ds_in, v_in, f_max_in
    integer :: i

    ds = ds_in
    dk = 2.*pi*f_max_in/v_in/100.
    e = nint(p_fit_in)
    k(1,1:e) = (/ ((i-1)*dk, i=1,e) /)

  end subroutine

  double precision function max_rel_err(a)

    real, intent(in) :: a(4)
    double precision :: oper(100)
    integer :: i, j

    max_rel_err = maxval(abs(1.+2*(a(1)*(cos(1.*k(1,1:e)*ds)-1.) + a(2)*(cos(2.*k(1,1:e)*ds)-1.) &
                                 + a(3)*(cos(3.*k(1,1:e)*ds)-1.) + a(4)*(cos(4.*k(1,1:e)*ds)-1.)) &
                                 /ds/ds/k(1,1:e)/k(1,1:e)))

  end function

  subroutine neighbor(a,an)
  
    real :: a(4), an(4), r
    integer :: i

    do while (.true.)
      call random_number(r)
      an(2) = a(2) + 0.01*c8(2)*(r-0.5)
      if (an(2) < 0.) then
        exit
      endif
    enddo

    do while (.true.)
      call random_number(r)
      an(3) = a(3) + 0.01*c8(3)*(r-0.5)
      if (an(3) > 0. .and. abs(an(3)) <= abs(an(2))) then
        exit
      endif
    enddo

    do while (.true.)
      call random_number(r)
      an(4) = a(4) + 0.01*c8(4)*(r-0.5)
      if (an(4) < 0. .and. abs(an(4)) <= abs(an(3))) then
        exit
      endif
    enddo

    an(1) = 1.-4.*an(2)-9.*an(3)-16.*an(4)

  end subroutine 

end module
